from ._exceptions import (
    APIError,
    AnonymousConnectionError,
    AnonymousSessionError,
    DumbFucksBeware,
    CDNError,
    InvalidAuthTokenError,
    InvalidChatLine,
    InvalidMessageError,
)
